import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()


### Get Number of Chains in Collection
_Get_NOC          = _spirit.Collection_Get_NOC
_Get_NOC.argtypes = [ctypes.c_void_p]
_Get_NOC.restype  = ctypes.c_int
def Get_NOC(p_state):
    return int(_Get_NOC(p_state))

### Go to next chain in collection
_next_Chain          = _spirit.Collection_next_Chain
_next_Chain.argtypes = [ctypes.c_void_p]
_next_Chain.restype  = ctypes.c_int
def Next_Chain(p_state):
    _next_Chain(p_state)

### Go to previous chain in collection
_prev_Chain          = _spirit.Collection_prev_Chain
_prev_Chain.argtypes = [ctypes.c_void_p]
_prev_Chain.restype  = ctypes.c_int
def Prev_Chain(p_state):
    _prev_Chain(p_state)


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

import threading

###     We use a thread for PlayPause, so that KeyboardInterrupt can be forwarded to the CDLL call
###     We might want to think about using PyDLL and about a signal handler in the core library
###     see here: http://stackoverflow.com/questions/14271697/ctrlc-doesnt-interrupt-call-to-shared-library-using-ctypes-in-python

### SingleShot Iteration
_SingleShot          = _spirit.Simulation_SingleShot
_SingleShot.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int, 
                        ctypes.c_int, ctypes.c_int, ctypes.c_int]
_SingleShot.restype  = None
def SingleShot(p_state, method_type, solver_type, n_iterations=-1, n_iterations_log=-1, 
               idx_image=-1, idx_chain=-1):
    spiritlib.WrapFunction(_SingleShot, [ctypes.c_void_p(p_state), 
                                         ctypes.c_char_p(method_type.encode('utf-8')), 
                                         ctypes.c_char_p(solver_type.encode('utf-8')), 
                                         ctypes.c_int(n_iterations), ctypes.c_int(n_iterations_log), 
                                         ctypes.c_int(idx_image), ctypes.c_int(idx_chain)])
    # _SingleShot(ctypes.c_void_p(p_state), ctypes.c_char_p(method_type), 
    #             ctypes.c_char_p(solver_type), ctypes.c_int(n_iterations), 
    #             ctypes.c_int(n_iterations_log), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Play/Pause
_PlayPause          = _spirit.Simulation_PlayPause
_PlayPause.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_int, 
                       ctypes.c_int, ctypes.c_int, ctypes.c_int]
_PlayPause.restype  = None
def PlayPause(p_state, method_type, solver_type, n_iterations=-1, n_iterations_log=-1, 
              idx_image=-1, idx_chain=-1):
    spiritlib.WrapFunction(_PlayPause, [ctypes.c_void_p(p_state), 
                                        ctypes.c_char_p(method_type.encode('utf-8')), 
                                        ctypes.c_char_p(solver_type.encode('utf-8')), 
                                        ctypes.c_int(n_iterations), ctypes.c_int(n_iterations_log), 
                                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain)])
    # _PlayPause(ctypes.c_void_p(p_state), ctypes.c_char_p(method_type), 
    #            ctypes.c_char_p(solver_type), ctypes.c_int(n_iterations), 
    #            ctypes.c_int(n_iterations_log), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Stop All
_Stop_All           = _spirit.Simulation_Stop_All
_Stop_All.argtypes  = [ctypes.c_void_p]
_Stop_All.restype   = None
def Stop_All(p_state):
    _Stop_All(ctypes.c_void_p(p_state))

### Check if a simulation is running on a specific image
_Running_Image            = _spirit.Simulation_Running_Image
_Running_Image.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Running_Image.restype    = ctypes.c_bool
def Running_Image(p_state, idx_image=-1, idx_chain=-1):
    return bool(_Running_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                               ctypes.c_int(idx_chain)))

### Check if a simulation is running across a specific chain
_Running_Chain            = _spirit.Simulation_Running_Chain
_Running_Chain.argtypes   = [ctypes.c_void_p, ctypes.c_int]
_Running_Chain.restype    = ctypes.c_bool
def Running_Chain(p_state, idx_chain=-1):
    return bool(_Running_Chain(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))

### Check if a simulation is running on across the collection
_Running_Collection            = _spirit.Simulation_Running_Collection
_Running_Collection.argtypes   = [ctypes.c_void_p]
_Running_Collection.restype    = ctypes.c_bool
def Running_Collection(p_state):
    return bool(_Running_Collection(ctypes.c_void_p(p_state)))


### Check if any simulation running on any image of - or the entire - chain
_Running_Anywhere_Chain           = _spirit.Simulation_Running_Anywhere_Chain
_Running_Anywhere_Chain.argtypes  = [ctypes.c_void_p, ctypes.c_int]
_Running_Anywhere_Chain.restype   = ctypes.c_bool
def Running_Anywhere_Chain(p_state, idx_chain=-1):
    return bool(_Running_Anywhere_Chain(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))

### Check if any simulation running on any image or chain of - or the entire - collection
_Running_Anywhere_Collection            = _spirit.Simulation_Running_Anywhere_Collection
_Running_Anywhere_Collection.argtypes   = [ctypes.c_void_p]
_Running_Anywhere_Collection.restype    = ctypes.c_bool
def Running_Anywhere_Collection(p_state):
    return bool(_Running_Anywhere_Collection(ctypes.c_void_p(p_state)))


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

_Get_Magnetization          = _spirit.Quantity_Get_Magnetization
_Get_Magnetization.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float),
                               ctypes.c_int, ctypes.c_int]
_Get_Magnetization.restype  = None
def Get_Magnetization(p_state, idx_image=-1, idx_chain=-1):
    magnetization = (3*ctypes.c_float)()
    _Get_Magnetization(ctypes.c_void_p(p_state), magnetization,
                       ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return [float(i) for i in magnetization]

_Get_Topological_Charge          = _spirit.Quantity_Get_Topological_Charge
_Get_Topological_Charge.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Topological_Charge.restype  = ctypes.c_float
def Get_Topological_Charge(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_Topological_Charge(ctypes.c_void_p(p_state),
                       ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))


import spirit.spiritlib as spiritlib
import ctypes

from spirit.scalar import scalar

# Load Library
_spirit = spiritlib.LoadSpiritLibrary()

# The Bohr Magneton [meV / T]
_mu_B           = _spirit.Constants_mu_B
_mu_B.argtypes  = None
_mu_B.restype   = scalar
def mu_B():
    return _mu_B()

# The vacuum permeability [T^2 m^3 / meV]
_mu_0           = _spirit.Constants_mu_0
_mu_0.argtypes  = None
_mu_0.restype   = scalar
def mu_0():
    return _mu_0()

# The Boltzmann constant [meV / K]
_k_B            = _spirit.Constants_k_B
_k_B.argtypes   = None
_k_B.restype    = scalar
def k_B():
    return _k_B()

# Planck's constant [meV*ps / rad]
_hbar           = _spirit.Constants_hbar
_hbar.argtypes  = None
_hbar.restype   = scalar
def hbar():
    return _hbar()

# Millirydberg [mRy / meV]
_mRy            = _spirit.Constants_mRy
_mRy.argtypes   = None
_mRy.restype    = scalar
def mRy():
    return _mRy()

# Gyromagnetic ratio of electron [rad / (ps*T)]
_gamma          = _spirit.Constants_gamma
_gamma.argtypes = None
_gamma.restype  = scalar
def gamma():
    return _gamma()

# Electron g-factor [unitless]
_g_e            = _spirit.Constants_g_e
_g_e.argtypes   = None
_g_e.restype    = scalar
def g_e():
    return _g_e()

# Pi [rad]
_Pi            = _spirit.Constants_Pi
_Pi.argtypes   = None
_Pi.restype    = scalar
def Pi():
    return _Pi()


__version__ = "1.8.1"
__title__ = "Spirit"
__description__ = "Optimizations and Dynamics Framework for atomistic Spin systems"
__uri__ = "https://spirit-code.github.io"
__license__ = "MIT"
__author__ = "Gideon Mueller"
__email__ = "g.mueller@fz-juelich.de"



import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### State wrapper class to be used in 'with' statement
class State:
    """Wrapper Class for a Spirit State"""

    def __init__(self, configfile="", quiet=False):
        self.p_state = setup(configfile, quiet)

    def __enter__(self):
        return self.p_state

    def __exit__(self, exc_type, exc_value, traceback):
        delete(self.p_state)


### Setup State
_State_Setup = _spirit.State_Setup
_State_Setup.argtypes = [ctypes.c_char_p, ctypes.c_bool]
_State_Setup.restype = ctypes.c_void_p
def setup(configfile="", quiet=False):
    return _State_Setup(ctypes.c_char_p(configfile.encode('utf-8')), ctypes.c_bool(quiet))

### Delete State
_State_Delete = _spirit.State_Delete
_State_Delete.argtypes = [ctypes.c_void_p]
_State_Delete.restype = None
def delete(p_state):
    return _State_Delete(ctypes.c_void_p(p_state))


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()


### Generate homogeneous transition between two images of a chain
_Homogeneous             = _spirit.Transition_Homogeneous
_Homogeneous.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Homogeneous.restype     = None
def Homogeneous(p_state, idx_1, idx_2, idx_chain=-1):
    _Homogeneous(ctypes.c_void_p(p_state), ctypes.c_int(idx_1), ctypes.c_int(idx_2), 
                 ctypes.c_int(idx_chain))

### Add some temperature-scaled noise to a transition between two images of a chain
_Add_Noise_Temperature             = _spirit.Transition_Add_Noise_Temperature
_Add_Noise_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, 
                                      ctypes.c_int, ctypes.c_int]
_Add_Noise_Temperature.restype     = None
def Add_Noise_Temperature(p_state, temperature, idx_1, idx_2, idx_chain=-1):
    _Add_Noise_Temperature(ctypes.c_void_p(p_state), ctypes.c_float(temperature), 
                           ctypes.c_int(idx_1), ctypes.c_int(idx_2), ctypes.c_int(idx_chain))


import codecs
import os
import platform
import re
import shutil
import stat
import subprocess
import sys
import datetime

from distutils.util import get_platform
from setuptools import setup, Command
from wheel.bdist_wheel import bdist_wheel as bdist_wheel_


NAME = "spirit"
PACKAGES = ['spirit']
META_PATH = os.path.join("spirit", "__init__.py")
KEYWORDS = ["Spirit", "Spin Dynamics"]
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Science/Research",
    "Natural Language :: English",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: C",
    "Programming Language :: C++",
    "Programming Language :: Python",
    "Topic :: Scientific/Engineering",
    "Topic :: Software Development :: Libraries :: Python Modules",
]
INSTALL_REQUIRES = ["numpy"]

###############################################################################

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """
    Extract __*meta*__ from META_FILE.
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta),
        META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))

def get_git_commit_datetime():
    try:
        commit_hash = subprocess.check_output("git rev-parse HEAD", shell=True, stderr=subprocess.STDOUT).decode("utf-8").strip()
        commit_datetime = subprocess.check_output("git show -s --format=%ci "+commit_hash, shell=True, stderr=subprocess.STDOUT).decode("utf-8").strip()
        print(commit_datetime)
        datetime_object = datetime.datetime.strptime(commit_datetime, '%Y-%m-%d %H:%M:%S +%f')
        print("{:%Y%m%d%H%M%S}".format(datetime_object))
        return "{:%Y%m%d%H%M%S}".format(datetime_object)
    except subprocess.CalledProcessError as cpe:
        print(cpe.output)
        return "00000000000000"

import unittest
def my_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('test', pattern='*.py')
    return test_suite

class bdist_wheel(bdist_wheel_):
    def finalize_options(self):
        from sys import platform as _platform
        platform_name = get_platform()
        if _platform == "linux" or _platform == "linux2":
            # Linux
            platform_name = 'manylinux1_x86_64'
        
        bdist_wheel_.finalize_options(self)
        self.universal = True
        self.plat_name_supplied = True
        self.plat_name = platform_name


class CleanCommand(Command):
    """Custom clean command to tidy up the project root."""
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


if __name__ == "__main__":
    # If the environment variable SPIRIT_VERSION_SUFFIX is defined,
    # it is appended to the package version number.
    version_suffix = ""
    add_version_suffix = os.environ.get("SPIRIT_ADD_VERSION_SUFFIX", "")
    if add_version_suffix.lower() in ("yes", "true", "t", "1"):
        timepoint_string = get_git_commit_datetime()
        if timepoint_string == "00000000000000":
            timepoint_string = "{:%Y%m%d%H%M}".format(datetime.datetime.now())
        version_suffix = ".dev"+timepoint_string
        print("setup.py: package version suffix = ", version_suffix)

    # Setup the package info
    setup(
        name             = NAME,
        description      = find_meta("description"),
        long_description = read('README.md'),
        license          = find_meta("license"),
        url              = find_meta("uri"),
        version          = find_meta("version")+version_suffix,
        author           = find_meta("author"),
        author_email     = find_meta("email"),
        maintainer       = find_meta("author"),
        maintainer_email = find_meta("email"),
        keywords         = KEYWORDS,
        packages         = PACKAGES,
        classifiers      = CLASSIFIERS,
        install_requires = INSTALL_REQUIRES,
        package_data     = {
            'spirit': ['libSpirit.dylib', 'libSpirit.so', 'libSpirit.dll'],
        },
        cmdclass         = {'bdist_wheel': bdist_wheel, 'clean': CleanCommand},
        test_suite       = 'setup.my_test_suite',
    )


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

from spirit.scalar import scalar
from numpy import frombuffer, ndarray as np

### Get Chain index
_Get_Index          = _spirit.System_Get_Index
_Get_Index.argtypes = [ctypes.c_void_p]
_Get_Index.restype  = ctypes.c_int
def Get_Index(p_state):
    return int(_Get_Index(ctypes.c_void_p(p_state)))

### Get Chain number of images
_Get_NOS            = _spirit.System_Get_NOS
_Get_NOS.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_NOS.restype    = ctypes.c_int
def Get_NOS(p_state, idx_image=-1, idx_chain=-1):
    return int(_Get_NOS(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Get Pointer to Spin Directions
# NOTE: Changing the values of the array_view one can alter the value of the data of the state
_Get_Spin_Directions            = _spirit.System_Get_Spin_Directions
_Get_Spin_Directions.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Spin_Directions.restype    = ctypes.POINTER(scalar)
def Get_Spin_Directions(p_state, idx_image=-1, idx_chain=-1):
    nos = Get_NOS(p_state, idx_image, idx_chain)
    ArrayType = scalar*3*nos
    Data = _Get_Spin_Directions(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                                ctypes.c_int(idx_chain))
    array_pointer = ctypes.cast(Data, ctypes.POINTER(ArrayType))
    array = frombuffer(array_pointer.contents, dtype=scalar)
    array_view = array.view()
    array_view.shape = (nos, 3)
    return array_view

### Get total Energy
_Get_Energy          = _spirit.System_Get_Energy
_Get_Energy.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Energy.restype  = ctypes.c_float
def Get_Energy(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_Energy(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                             ctypes.c_int(idx_chain)))

# NOTE: excluded since there is no clean way to get the C++ pairs
### Get Energy array
# _Get_Energy_Array          = _spirit.System_Get_Energy_Array
# _Get_Energy_Array.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
#                               ctypes.c_int, ctypes.c_int]
# _Get_Energy_Array.restype  = None
# def Get_Energy_Array(p_state, idx_image=-1, idx_chain=-1):
#     Energies 
#     _Get_Energy_Array(ctypes.c_void_p(p_state), energies,
#                       ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Get Chain number of images
_Update_Data            = _spirit.System_Update_Data
_Update_Data.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Update_Data.restype    = None
def Update_Data(p_state, idx_image=-1, idx_chain=-1):
    _Update_Data(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Print Energy array
_Print_Energy_Array            = _spirit.System_Print_Energy_Array
_Print_Energy_Array.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Print_Energy_Array.restype    = None
def Print_Energy_Array(p_state, idx_image=-1, idx_chain=-1):
    _Print_Energy_Array(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))